# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rx/proto/rx.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11rx/proto/rx.proto\x12\x02rx\x1a\x1bgoogle/protobuf/empty.proto\"\x1d\n\x08Hardware\x12\x11\n\tprocessor\x18\x01 \x01(\t\":\n\x05Image\x12\x10\n\x08registry\x18\x01 \x01(\t\x12\x12\n\nrepository\x18\x02 \x01(\t\x12\x0b\n\x03tag\x18\x03 \x01(\t\"7\n\x06Remote\x12\x1e\n\x08hardware\x18\x01 \x01(\x0b\x32\x0c.rx.Hardware\x12\r\n\x05ports\x18\x02 \x03(\x05\"\"\n\x12\x45nvironmentManager\x12\x0c\n\x04name\x18\x01 \x01(\t\"q\n\x0b\x45nvironment\x12\x1a\n\x06remote\x18\x01 \x01(\x0b\x32\n.rx.Remote\x12\x18\n\x05image\x18\x02 \x01(\x0b\x32\t.rx.Image\x12,\n\x0c\x65nv_managers\x18\x03 \x03(\x0b\x32\x16.rx.EnvironmentManager\"?\n\x05\x44\x65lta\x12\x10\n\x08\x61\x64\x64_path\x18\x01 \x03(\t\x12\x0f\n\x07\x61\x64\x64_dir\x18\x02 \x03(\t\x12\x13\n\x0bremove_path\x18\x03 \x03(\t\"Q\n\x13\x44ockerImageProgress\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\r\n\x05total\x18\x03 \x01(\x03\x12\x0f\n\x07\x63urrent\x18\x04 \x01(\x03\"%\n\rSubscribeInfo\x12\x14\n\x0cpayment_link\x18\x01 \x01(\t\"7\n\x06Result\x12\x1c\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x0e.rx.StatusCode\x12\x0f\n\x07message\x18\x02 \x01(\t\"H\n\x0bRsyncSource\x12\x12\n\nmachine_id\x18\x01 \x01(\x03\x12\x11\n\tdirectory\x18\x02 \x01(\t\x12\x12\n\npublic_key\x18\x03 \x01(\x0c\")\n\x10RsyncDestination\x12\x15\n\rdaemon_module\x18\x01 \x01(\t\":\n\tExecution\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\x12\x10\n\x08start_ts\x18\x02 \x01(\x03\x12\x0e\n\x06\x65nd_ts\x18\x03 \x01(\x03\"-\n\x0fGenericResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\"x\n\x0b\x45xecRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x0c\n\x04\x61rgv\x18\x02 \x03(\t\x12)\n\x0crsync_source\x18\x03 \x01(\x0b\x32\x0f.rx.RsyncSourceB\x02\x18\x01\x12\x0b\n\x03\x63wd\x18\x04 \x01(\t\x12\r\n\x05stdin\x18\x05 \x01(\x0c\"\x89\x01\n\x0c\x45xecResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x14\n\x0c\x65xecution_id\x18\x02 \x01(\t\x12\x0e\n\x06stdout\x18\x03 \x01(\x0c\x12\x0e\n\x06stderr\x18\x04 \x01(\x0c\x12\x14\n\x0coutput_files\x18\x05 \x03(\t\x12\x11\n\texit_code\x18\x06 \x01(\x05\"a\n\x18GetSubscribeInfoResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12)\n\x0esubscribe_info\x18\x02 \x01(\x0b\x32\x11.rx.SubscribeInfo\"?\n\x0fGetUserResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x10\n\x08username\x18\x02 \x01(\t\"/\n\x17GetWorkspaceInfoRequest\x12\x14\n\x0cworkspace_id\x18\x02 \x01(\t\"\x8b\x01\n\x18GetWorkspaceInfoResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\r\n\x05state\x18\x02 \x01(\t\x12\x1e\n\x07history\x18\x03 \x03(\x0b\x32\r.rx.Execution\x12$\n\x0b\x65nvironment\x18\x04 \x01(\x0b\x32\x0f.rx.Environment\"o\n\x0bInitRequest\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12%\n\x0crsync_source\x18\x02 \x01(\x0b\x32\x0f.rx.RsyncSource\x12#\n\ntarget_env\x18\x03 \x01(\x0b\x32\x0f.rx.Environment\"\x7f\n\x0cInitResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12(\n\nrsync_dest\x18\x02 \x01(\x0b\x32\x14.rx.RsyncDestination\x12\x13\n\x0bworker_addr\x18\x03 \x01(\t\x12\x14\n\x0cworkspace_id\x18\x04 \x01(\t\"*\n\x12InstallDepsRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\"A\n\x13InstallDepsResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x0e\n\x06stdout\x18\x02 \x01(\x0c\"9\n\x0bKillRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x14\n\x0c\x65xecution_id\x18\x02 \x01(\t\"\'\n\x0fUnfreezeRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\"C\n\x10UnfreezeResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x13\n\x0bworker_addr\x18\x02 \x01(\t\"&\n\x12SetUsernameRequest\x12\x10\n\x08username\x18\x01 \x01(\t\"1\n\x0bStopRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x0c\n\x04save\x18\x02 \x01(\x08\"t\n\x0cStopResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12.\n\rpush_progress\x18\x02 \x01(\x0b\x32\x17.rx.DockerImageProgress\x12\x18\n\x05image\x18\x03 \x01(\x0b\x32\t.rx.Image\")\n\x11WorkerInitRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\"`\n\x12WorkerInitResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12.\n\rpull_progress\x18\x02 \x01(\x0b\x32\x17.rx.DockerImageProgress*\x82\x01\n\nStatusCode\x12\x06\n\x02OK\x10\x00\x12\x0b\n\x07UNKNOWN\x10\x01\x12\x0b\n\x07INVALID\x10\x02\x12\r\n\tNOT_FOUND\x10\x03\x12\x10\n\x0cUNAUTHORIZED\x10\x04\x12\x19\n\x15SUBSCRIPTION_REQUIRED\x10\x05\x12\n\n\x06\x45\x41GAIN\x10\x0b\x12\n\n\x05MOVED\x10\xad\x02\x32\xf5\x01\n\x10\x45xecutionService\x12\x39\n\x04Init\x12\x15.rx.WorkerInitRequest\x1a\x16.rx.WorkerInitResponse\"\x00\x30\x01\x12\x42\n\x0bInstallDeps\x12\x16.rx.InstallDepsRequest\x1a\x17.rx.InstallDepsResponse\"\x00\x30\x01\x12/\n\x04\x45xec\x12\x0f.rx.ExecRequest\x1a\x10.rx.ExecResponse\"\x00(\x01\x30\x01\x12\x31\n\x04Kill\x12\x0f.rx.KillRequest\x1a\x16.google.protobuf.Empty\"\x00\x32\xbc\x04\n\x0cSetupService\x12\x42\n\x11\x43heckSubscription\x12\x16.google.protobuf.Empty\x1a\x13.rx.GenericResponse\"\x00\x12J\n\x10GetSubscribeInfo\x12\x16.google.protobuf.Empty\x1a\x1c.rx.GetSubscribeInfoResponse\"\x00\x12\x38\n\x07GetUser\x12\x16.google.protobuf.Empty\x1a\x13.rx.GetUserResponse\"\x00\x12O\n\x10GetWorkspaceInfo\x12\x1b.rx.GetWorkspaceInfoRequest\x1a\x1c.rx.GetWorkspaceInfoResponse\"\x00\x12+\n\x04Init\x12\x0f.rx.InitRequest\x1a\x10.rx.InitResponse\"\x00\x12\x39\n\x08Unfreeze\x12\x13.rx.UnfreezeRequest\x1a\x14.rx.UnfreezeResponse\"\x00\x30\x01\x12<\n\x0bSetUsername\x12\x16.rx.SetUsernameRequest\x1a\x13.rx.GenericResponse\"\x00\x12-\n\x04Stop\x12\x0f.rx.StopRequest\x1a\x10.rx.StopResponse\"\x00\x30\x01\x12<\n\x0bUnsubscribe\x12\x16.google.protobuf.Empty\x1a\x13.rx.GenericResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rx.proto.rx_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EXECREQUEST.fields_by_name['rsync_source']._options = None
  _EXECREQUEST.fields_by_name['rsync_source']._serialized_options = b'\030\001'
  _STATUSCODE._serialized_start=2311
  _STATUSCODE._serialized_end=2441
  _HARDWARE._serialized_start=54
  _HARDWARE._serialized_end=83
  _IMAGE._serialized_start=85
  _IMAGE._serialized_end=143
  _REMOTE._serialized_start=145
  _REMOTE._serialized_end=200
  _ENVIRONMENTMANAGER._serialized_start=202
  _ENVIRONMENTMANAGER._serialized_end=236
  _ENVIRONMENT._serialized_start=238
  _ENVIRONMENT._serialized_end=351
  _DELTA._serialized_start=353
  _DELTA._serialized_end=416
  _DOCKERIMAGEPROGRESS._serialized_start=418
  _DOCKERIMAGEPROGRESS._serialized_end=499
  _SUBSCRIBEINFO._serialized_start=501
  _SUBSCRIBEINFO._serialized_end=538
  _RESULT._serialized_start=540
  _RESULT._serialized_end=595
  _RSYNCSOURCE._serialized_start=597
  _RSYNCSOURCE._serialized_end=669
  _RSYNCDESTINATION._serialized_start=671
  _RSYNCDESTINATION._serialized_end=712
  _EXECUTION._serialized_start=714
  _EXECUTION._serialized_end=772
  _GENERICRESPONSE._serialized_start=774
  _GENERICRESPONSE._serialized_end=819
  _EXECREQUEST._serialized_start=821
  _EXECREQUEST._serialized_end=941
  _EXECRESPONSE._serialized_start=944
  _EXECRESPONSE._serialized_end=1081
  _GETSUBSCRIBEINFORESPONSE._serialized_start=1083
  _GETSUBSCRIBEINFORESPONSE._serialized_end=1180
  _GETUSERRESPONSE._serialized_start=1182
  _GETUSERRESPONSE._serialized_end=1245
  _GETWORKSPACEINFOREQUEST._serialized_start=1247
  _GETWORKSPACEINFOREQUEST._serialized_end=1294
  _GETWORKSPACEINFORESPONSE._serialized_start=1297
  _GETWORKSPACEINFORESPONSE._serialized_end=1436
  _INITREQUEST._serialized_start=1438
  _INITREQUEST._serialized_end=1549
  _INITRESPONSE._serialized_start=1551
  _INITRESPONSE._serialized_end=1678
  _INSTALLDEPSREQUEST._serialized_start=1680
  _INSTALLDEPSREQUEST._serialized_end=1722
  _INSTALLDEPSRESPONSE._serialized_start=1724
  _INSTALLDEPSRESPONSE._serialized_end=1789
  _KILLREQUEST._serialized_start=1791
  _KILLREQUEST._serialized_end=1848
  _UNFREEZEREQUEST._serialized_start=1850
  _UNFREEZEREQUEST._serialized_end=1889
  _UNFREEZERESPONSE._serialized_start=1891
  _UNFREEZERESPONSE._serialized_end=1958
  _SETUSERNAMEREQUEST._serialized_start=1960
  _SETUSERNAMEREQUEST._serialized_end=1998
  _STOPREQUEST._serialized_start=2000
  _STOPREQUEST._serialized_end=2049
  _STOPRESPONSE._serialized_start=2051
  _STOPRESPONSE._serialized_end=2167
  _WORKERINITREQUEST._serialized_start=2169
  _WORKERINITREQUEST._serialized_end=2210
  _WORKERINITRESPONSE._serialized_start=2212
  _WORKERINITRESPONSE._serialized_end=2308
  _EXECUTIONSERVICE._serialized_start=2444
  _EXECUTIONSERVICE._serialized_end=2689
  _SETUPSERVICE._serialized_start=2692
  _SETUPSERVICE._serialized_end=3264
# @@protoc_insertion_point(module_scope)
